package cv08.server;

import java.util.*;

/**
 * @author Karel Cemus
 */
public class Room {

    /** seznam vsech mistnosti na serveru */
    private static final Map<String, Room> rooms = new HashMap<String, Room>();

    /** nazev mistnosti */
    private final String name;

    /** aktualne pritomni uzivatele */
    private final Set<Client> clients = new HashSet<Client>();

    /** zalozit mistnost s danym nazvem */
    public static boolean createRoom( String name ) {
        // mistnost existuje, nevytvaret
        if ( rooms.containsKey( name ) ) return false;
        // pokud mistnost neexistuje, tak ji vytvorime
        rooms.put( name, new Room( name ) );
        return true;
    }

    public static Set<String> list() {
        return rooms.keySet();
    }

    /** vraci mistnost odpovidajici pozadovanemu nazvu */
    public static Room get( String name ) {
        return rooms.get( name );
    }

    public Room( String name ) {
        this.name = name;
    }

    /** odeslat zpravu vsem uzivatelum v mistnosti */
    public void send( String sender, String message ) {
        // vypsat zpravu
        System.out.println( String.format( "[%s] %s: %s", name, sender, message ) );
        // prevest zpravu na prikaz
        String command = String.format( "RECV %s %s %s", name, sender, message );
        // zpracovat zpravu zaslanou do mistnosti
        for ( Client client : clients ) {
            if ( client.send( command ) ) {
                System.out.println( String.format( "Message send to %s in the room %s.", client.getNickname(), name ) );
            } else {
                System.out.println( String.format( "Message for %s wasn't send.", client.getNickname() ) );
                rooms.remove( client );
            }
        }
    }

    /** odeslat zpravu vsem uzivatelum v mistnosti */
    public void send( Client sender, String message ) {

        if ( !clients.contains( sender ) ) {
            sender.send( "ERR 06 Message ignored. Sender is not in the room." );
        } else {
            send( sender.getNickname(), message );
        }
    }

    /** uzivatel chce vstoupit do mistnosti */
    public void enter( Client client ) {
        // uzivatel vstoupil do mistnosti
        clients.add( client );
    }

    /** vraci zdali je klient v mistnosti */
    public boolean contains( Client client ) {
        return clients.contains( client );
    }

    /** uzivatel chce odejit z mistnosti */
    public void leave( Client client ) {
        // uzivatel mistnost opustil
        clients.remove( client );
    }

    public String getName() {
        return name;
    }
}
